class Paths:
    #TODO Traslate texts in German
    # Login Procedue
    ACCEPT_COOKIES = "/html/body/div[2]/div/div/div/div[2]/button[1]"
    LOGIN_BTN = '//button[@class="sqdOP  L3NKy   y3zKF     "]/div'
    USERNAME_INPUT = '//input[@name="username"]'
    PASSWORD_INPUT = '//input[@name="password"]'
    SECURITY_CODE = '//input[@name="verificationCode"]'
    SECURITY_CODE_BTN = '//button[@class="sqdOP  L3NKy   y3zKF     "]'
    NO_NOTIFICATIONS_BTN = '/html/body/div[4]/div/div/div/div[3]/button[2]'
    # Nav to User Procedure
    INCORRECT_USERNAME_ALERT = '//p[@role="alert" and @id="slfErrorAlert"]'
    INCORRECT_PASSWORD_ALERT = '//div[@class="piCib"]'
    INCORRECT_PASSWORD_ALERT_BTNS = '{}//button[contains(@class, "aOOlW")]'
    ALERT = '//*[@id="slfErrorAlert" or @id="twoFactorErrorAlert"]'
    PAGE_NOT_FOUND = '//h2[@class="_7UhW9      x-6xq    qyrsm KV-D4          uL8Hv     l4b0S    "]'
    PRIVATE_ACCOUNT_ALERT = '//h2[@class="rkEop"]'
    # Followers Procedure
    FOLLOWERS_BTN = '//li[@class=" LH36I"]//descendant::a'
    FOLLOWERS_LIST_MAIN = '//main[@role="main"]'
    FOLLOWERS_LIST = '//ul[@class=" jjbaz _6xe7A"]'
    FOLLOWER_USER_DIV = '{}//li'.format(FOLLOWERS_LIST)
    FOLLOWER_COUNT = '{}//span[@class="g47SY lOXF2"]'.format(FOLLOWERS_BTN)
    # Send DM Procedure
    DM_TEXT_AREA = '//div[@class="X3a-9"]//descendant::textarea'
    SEND_DM_BTN = '//div[@class="X3a-9"]//descendant::button'
    # Check Login Status Procedure
    NAV_BAR = '//div[@data-testid="mobile-nav-logged-in" and @class="BvyAW"]'
    # GENERAL
    BUTTON = '//button[text()="{}"]'


class ClientUrls:
    LOGIN_URL='https://www.instagram.com/accounts/login/'
    NAV_USER='https://www.instagram.com/{}/'
    SEARCH_TAGS='https://www.instagram.com/explore/tags/{}/'
    FOLLOWERS_URL = 'https://www.instagram.com/{}/followers/'
    HOME_URL =  'https://instagram.com/'


"""This module contains common errors and exceptions raised by the InstaClient"""
from selenium.common.exceptions import WebDriverException

class InstaClientError(Exception):
    """Base InstaClient Exception

    Attributes:
        message -- explanation of the error
    """

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'{self.message}'


class InvaildHostError(InstaClientError):
    """Raised when trying to pass an incorrect host index in InstaClient.__init__()

    Args:
        host_int:int: The driver int variable passed to InstaClient()
    """
    def __init__(self, host_int:int):
        self.host_int = host_int
        super().__init__(message='This integer does not refer to any host')

    def __str__(self):
        return f'{self.host_int} -> {self.message}'


class InvaildDriverError(InstaClientError):
    """Raised when trying to pass an incorrect driver index in InstaClient.__init__()

    Args:
        driver_int:int: The driver int variable passed to InstaClient()
    """
    def __init__(self, driver_int:int):
        self.driver_int = driver_int
        super().__init__(message='This integer does not refer to any driver')

    def __str__(self):
        return f'{self.driver_int} -> {self.message}'


class InvalidUserError(InstaClientError):
    """Raised when searching for an incorrect user's username or when trying to login with an unexisting account's username
    
    Args:
        user:str: The user's username that caused the exception
    """
    def __init__(self, username:str):
        self.username = username
        super().__init__(message='The username was not recognized. Operation to navigate to the user was unsuccessful.')

    def __str__(self):
        return f'{self.username} -> {self.message}'


class InvaildPasswordError(InstaClientError):
    """Raised when trying to log in with an incorrect password
    
    Args:
        password:str: The passwod that caused the exception
    """
    def __init__(self, password:str):
        self.password = password
        super().__init__(message='The password used to attempt login is incorrect. Check the password.')


class SecurityCodeNecessary(InstaClientError):
    """Raised if security code is necessary to log in"""
    def __init__(self):
        super().__init__(message='The 2FA security code is required. The Security Code has been sent to the user\'s phone number or Authenticator App.')


class InvalidSecurityCodeError(InstaClientError):
    """Raised if security code inputted by the user is invalid"""
    def __init__(self):
        super().__init__(message='The used security code is invalid. Please try entering the code correctly or ask the user to input one of their backup codes')


class InvaildTagError(InstaClientError):
    """Raised when searching for an incorrect tag
    
    Args:
        tag:str: The tag that raised the exception"""
    def __init__(self, tag):
        self.tag = tag
        super().__init__(message='The tag you searched for does not exist.')

    def __str__(self):
        return f'{self.tag} -> {self.message}'


class PrivateAccountError(InstaClientError):
    """Raise when trying to access a private account's followers
    Args: 
        user:str: The username of the private account that caused the error
    """
    def __init__(self, user):
        self.user = user
        super().__init__(message='Getting this account\'s followers is impossible as this account is private')

    def __str__(self):
        return f'{self.user} -> {self.message}'
